@@ -247,6 +247,10 @@ class GraphPostableMixin(ILocalResource):
 
         return self._find_created_query(new_graph, query)
 
+    def check_new(self, created):
+        """Proxy to :func:`check_new` that can be overrided by children classes."""
+        return check_new(self.get_state(), created)
+
     def check_posted_graph(self, parameters, created, new_graph):
         """Check whether `new_graph` is acceptable to post on this resource.
 
@@ -270,7 +274,7 @@ class GraphPostableMixin(ILocalResource):
         # unused argument 'new_graph' #pylint: disable=W0613
         diag = Diagnosis("check_posted_graph")
         if isinstance(created, URIRef):
-            if not check_new(self.get_state(), created):
+            if not self.check_new(created):
                 diag.append("URI already in use <%s>" % created)
         return diag
 
@@ -593,7 +597,7 @@ class WithCardinalityMixin(ILocalResource):
             if maxc is not None and nbp > maxc:
                 diag.append("Property <%s> to <%s> should have at most "
                               "%s subjects; it has %s"
-                              % (p, uri, minc, nbp))
+                              % (p, uri, maxc, nbp))
 
         new_graph_objects = new_graph.objects
         for p, minc, maxc in cls.__get_cardinality_out():
@@ -605,7 +609,7 @@ class WithCardinalityMixin(ILocalResource):
             if maxc is not None and nbp > maxc:
                 diag.append("Property <%s> of <%s> should have at most "
                               "%s objects; it has %s"
-                              % (p, uri, minc, nbp))
+                              % (p, uri, maxc, nbp))
 
         return diag
 
